xquery version "3.1";

module namespace index-query="https://mollenhauer-edition.de/ns/index-query";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace index="https://mollenhauer-edition.de/index";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../../lang.xqm";

declare variable $index-query:indexes := collection($config:register);

declare function index-query:build-info($id)
{
    let $lang := request:get-parameter("lang", "")
    return
    for $entry in $index-query:indexes//index:entry[matches(@index:ref, $id)]
    let $type := $entry/ancestor::index:index/@type
    return
        element div {
            attribute class { "editorial-content entity hidden" },
            attribute data-ref { $id },
            attribute data-type { $type },
            element h4 {
                attribute class {"index-entry-title"},
                element span {
                    if ($entry/index:name[@xml:lang]) then
                        $entry/index:name[@xml:lang = $lang][1]/text()
                    else
                        $entry/index:name/text()
                },
                element button {
                    attribute class {"button-close index-entry-dismiss"},
                    attribute aria-label {"Close"},
                    element span {
                        attribute class {"icon-inverse icon-xmark"},
                        attribute aria-hidden {"true"}
                    }
                }
            },
            element dl {
                attribute class {"index-entry-body"},
                for $datum in $entry/*[self::index:artist] return
                    (element dt {
                        attribute class {"index-entry-label"},
                        lang:translate("index" || local-name($datum))
                    },
                    element dd {
                        attribute class {"index-entry-value"},
                        try {
                            xs:date($datum/text()) => format-date("[D]. [MNn] [Y]", "de", (), ())
                        }
                        catch * {
                            $datum/text()
                        }
                    }),
                for $datum in $entry/index:reproduction return
                    (element dt {
                        attribute class {"index-entry-label"},
                        lang:translate("index" || local-name($datum))
                    },
                    element dd {
                        attribute class {"index-entry-value"},
                            element a {
                            attribute href { $datum/@index:ref },
                            attribute target { "_blank" },
                            $datum/text(),
                            " ",
                            element span {
                                attribute class { "fa fa-external-link-alt" }
                            }
                        }
                    }),
                for $datum in $entry/*[
                    not(self::index:references
                        or self::index:name
                        or self::index:external
                        or self::index:reproduction
                        or self::index:repository
                        or self::index:artist
                        or self::index:keyword)
                ] return
                    (element dt {
                        attribute class {"index-entry-label"},
                        lang:translate("index" || local-name($datum))
                    },
                    element dd {
                        attribute class {"index-entry-value"},
                        try {
                            xs:date($datum/text()) => format-date("[D]. [MNn] [Y]", "de", (), ())
                        }
                        catch * {
                            $datum/text()
                        }
                    }),
                if ($entry/index:keyword) then
                    (element dt {
                        attribute class {"index-entry-label"},
                        lang:translate("indexkeyword")
                    },
                    for $datum in $entry/index:keyword return
                        element dd {
                            attribute class {"index-entry-value"},
                            $datum/text()
                        })
                else
                    (),
                for $datum in $entry/index:reproduction | $entry/index:external return
                    (element dt {
                        attribute class {"index-entry-label"},
                        lang:translate("index" || local-name($datum))
                    },
                    element dd {
                        attribute class {"index-entry-value"},
                            element a {
                            attribute href { $datum/@index:ref },
                            attribute target { "_blank" },
                            $datum/text(),
                            " ",
                            element span {
                                attribute class { "fa fa-external-link-alt" }
                            }
                        }
                    }),
            if ($entry/@index:ref) then
                (element span {
                    attribute class { "index-entry-index-link" },
                    element a {
                        attribute href {
                            "register.html?entity="
                            || substring-after($id, ":")
                            || "&amp;register=" || $entry/ancestor::index:index/@type
                        },
                        attribute target { "_blank" },
                        (lang:translate("indexlink")),
                        " ",
                        element span {
                            attribute class { "fa fa-external-link-alt" }
                        }
                    }
                })
            else
                ()
            }
        }
};
