xquery version "3.1";

 (:~
 : This module generates register as TEI-XML for person entities from the editions-TEI
 : and for artworks from lido-XML
 : @author Florian Barth
 : @version 1.0
 : @since 1.0
 :)


module namespace kmg-register-transformation="https://mollenhauer-edition.de/ns/register-transformation";

declare namespace index="https://mollenhauer-edition.de/index";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace pnd="http://textgrid.info/namespaces/vocabularies/pnd";
declare namespace lido="http://www.lido-schema.org";
declare namespace gndo="https://d-nb.info/standards/elementset/gnd#";
declare namespace entity-xml="https://sub.uni-goettingen.de/met/standards/entity-xml#";

import module namespace functx="http://www.functx.com";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";

declare function kmg-register-transformation:get-normdata-entities($entities as xs:string*, $register as xs:string, $uri as xs:string)
as element(index:entry)*{
    for $entity in $entities return
        if (not(starts-with($entity, 'gnd:'))) then
            kmg-register-transformation:get-kmg-entities($entity, $register)
        else
            kmg-register-transformation:get-gnd-entities($entity, $register, $uri)
};

declare function kmg-register-transformation:get-gnd-entities($entity as xs:string*, $register as xs:string, $uri as xs:string)
as element(index:entry)*
{
    let $stripped_id := tokenize($entity, ":")[2]
    return
        try{
            let $data := json-doc("https://lobid.org/gnd/" || $stripped_id || ".json")
            return
                element index:entry {
                    attribute index:ref { $entity },
                    element index:name { $data("preferredName") },
                    if (exists($data("dateOfBirth"))) then
                        element index:birth { $data("dateOfBirth")(1) }
                    else (),
                    if (exists($data("dateOfDeath"))) then
                        element index:death { $data("dateOfDeath")(1) }
                    else (),
                    if (exists($data("firstComposer"))) then
                        element index:artist { $data("firstComposer")(1)("label") }
                    else (),
                    if (exists($data("dateOfProduction"))) then
                        element index:date { $data("dateOfProduction")(1) }
                    else (),
                    if (
                        not(exists($data("dateOfBirth"))
                        and exists($data("dateOfDeath")))
                        and exists($data("udkCode"))
                    ) then
                        element index:udk { $data("udkCode") }
                    else (),
                    if ($register = "music") then
                        kmg-register-transformation:get-music-externals($entity, $uri)
                    else (),
                    element index:references {}
                }
        } catch * {
            element index:error {
                attribute index:ref { $entity },
                element index:name { "Could not retrieve data for this id." },
                element index:references {}
            }
        }
};

declare function kmg-register-transformation:get-music-externals($entity as xs:string, $uri as xs:string) {
    let $doc := doc($config:tei || $uri)
    let $externals := distinct-values($doc//*[@ref=$entity]/tei:ref/@target)
    return
        for $external in $externals return
            element index:external {
                attribute index:ref { $external },
                kmg-register-transformation:trim-uri($external)
            }
};

declare function kmg-register-transformation:get-kmg-entities($entity as xs:string*, $register as xs:string)
as element(index:entry)*
{
    let $stripped_id := tokenize($entity, "#")[2]
    let $reference-doc := doc($config:aux || "46ntv.xml")
    let $reference-entity := $reference-doc//entity-xml:person[@xml:id = $stripped_id]
    let $name := string-join(($reference-entity//gndo:preferredName/gndo:surname, $reference-entity//gndo:preferredName/gndo:forename), ", ")
    return
        element index:entry {
            attribute index:ref { $entity },
            attribute index:source { "kmg" },
            element index:name {$name},
            element index:references {}
        }
};

declare function kmg-register-transformation:trim-uri($uri as xs:string) as xs:string {
    replace($uri, "https?://", "") => substring-before("/")
};

declare function kmg-register-transformation:get-lido-entities($entities as xs:string*, $uri as xs:string)
as element(index:entry)* {
    for $entity in $entities
    let $doc := doc($config:lido || substring-after($entity, ":") || ".xml")
    return
        if (not($doc)) then
            ()
        else
        (: creation is not returned, it is a shortcut for retrieving metadata :)
        let $creation := $doc//lido:event[lido:eventType/lido:term = "Daseinsbeginn"]
        let $title := $doc//lido:titleSet/lido:appellationValue
        let $artist := $creation/lido:eventActor
        let $artist-name := $artist//lido:displayActorInRole/text()
        let $artist-ref := $artist//lido:actorID[@lido:type = "http://terminology.lido-schema.org/identifier_type/uri"]/text()
        let $categories := $doc//lido:objectClassificationWrap/lido:objectWorkTypeWrap/lido:objectWorkType/lido:term/text()
        let $date := $creation//lido:displayDate/text()
        let $place := $creation//lido:displayPlace/text()
        let $measurements := $doc//lido:displayObjectMeasurements/text()
        let $material-technique := $creation//lido:displayMaterialsTech/text()
        let $repository := $doc//lido:repositorySet/lido:repositoryName/lido:legalBodyName/lido:appellationValue/text()
        let $repository-uri := $doc//lido:repositorySet/lido:repositoryName/lido:legalBodyWeblink/text()
        let $workId := $doc//lido:repositorySet/lido:workID/text()
        let $reproductions := $doc//lido:relatedWork//lido:objectID[@lido:type = "http://terminology.lido-schema.org/identifier_type/uri"]/text()
        let $externals :=
            for $description in $doc//lido:relatedWork[not(lido:objectID/@lido:type != "http://terminology.lido-schema.org/identifier_type/uri")] return
            $description//lido:objectWebResource/text()
        let $keywords := $doc//lido:descriptiveMetadata/lido:objectClassificationWrap/lido:classificationWrap/lido:classification/lido:term/text()
        return
            element index:entry {
                attribute index:ref { $entity },
                if ($title) then
                    for $titleVariant in $title return
                        element index:name {
                            attribute xml:lang { $titleVariant/@xml:lang },
                            $titleVariant/text() }
                    else (),
                if ($artist-name) then element index:artist {
                    attribute index:ref { $artist-ref },
                    $artist-name
                    } else (),
                if ($categories) then
                    for $category in $categories return
                        element index:category { $category }
                    else (),
                if ($date) then element index:date { $date } else (),
                if ($place) then element index:place { $place } else (),
                if ($measurements) then element index:measurements { $measurements } else (),
                if ($material-technique) then element index:material { $material-technique } else (),
                if ($repository) then element index:repository {
                    attribute index:ref { $repository-uri },
                    $repository
                    } else (),
                if ($workId) then element index:workId { $workId } else (),
                if ($reproductions) then
                    for $reproduction in $reproductions return
                    element index:reproduction {
                        attribute index:ref { $reproduction },
                        kmg-register-transformation:trim-uri($reproduction)
                        }
                else (),
                if ($externals) then
                    for $external in $externals return
                    element index:external {
                        attribute index:ref { $external },
                        kmg-register-transformation:trim-uri($external)
                        }
                else (),
                if ($keywords) then
                    for $keyword in $keywords return
                        element index:keyword { $keyword }
                else (),
                element index:references {}
            }
};

declare function kmg-register-transformation:get-new-entities($entities as xs:string*, $register as xs:string, $uri as xs:string){
    if ($register = "art") then
        kmg-register-transformation:get-lido-entities($entities, $uri)
    else
        kmg-register-transformation:get-normdata-entities($entities, $register, $uri)
};

declare function kmg-register-transformation:create-register($register as xs:string){
    let $body :=
        element {fn:QName("https://mollenhauer-edition.de/index", "index")} {
            attribute xml:lang { "de" },
            attribute type { $register },
            element head {
                element title { "Index of " || $register || " entities"},
                element publisher {
                    element name { "Klaus Mollehnauer Gesamtausgabe" }
                }
            },
            element body {}
        }
    let $write-register := xmldb:store($config:register, $register || ".xml", $body)
    return
        util:log-system-out(util:system-time() || ": created empty register file for " || $register || " entities.")
};

declare function kmg-register-transformation:check-availability($register as xs:string){
    if (exists(doc($config:register || $register || ".xml")))
    then
        util:log-system-out(util:system-time() || ": found register for " || $register || " entities.")
    else
        kmg-register-transformation:create-register($register)
};

declare function kmg-register-transformation:update-registers($uri as xs:string){
    let $registers := ("art",
                    "brandname",
                    "music",
                    "org",
                    "periodica",
                    "person")
    let $updates :=
        for $register in $registers
            let $check := kmg-register-transformation:check-availability($register)
            let $update := kmg-register-transformation:update-register($uri, $register)
            let $references := kmg-register-transformation:update-references($uri, $register)
        return
            true()
    return
        util:log-system-out(util:system-time() || ": register creation completed. Returning to main process.")
};
declare function kmg-register-transformation:remove-resolved-error($doc as node()*)
{
    for $error in $doc//index:error return
        if ($error/@index:ref = $doc//index:entry/@index:ref) then
            update delete $error
        else
            ()
};

declare function kmg-register-transformation:update-references($uri as xs:string, $register as xs:string){
    let $doc := doc($config:tei || $uri)
    let $all-entities := distinct-values($doc//tei:text//tei:name[@type=$register]/@ref | $doc//tei:text//tei:rs[@type=$register]/@ref)
    let $entities :=
        for $entity in $all-entities return
            tokenize($entity, " ")
    let $register-doc := doc($config:register || $register || ".xml")
    let $register-entities := $register-doc//index:entry
    for $entity in $entities
        let $entry := $register-entities[@index:ref = $entity]
        return
            if ($entry//index:reference/@index:ref = $uri) then
                ()
            else
                update insert element index:reference {
                    attribute index:ref { $uri }
                } into $entry/index:references
};

declare function kmg-register-transformation:update-register($uri as xs:string, $register as xs:string){
    let $doc := doc($config:tei || $uri)
    let $all-entities :=
        distinct-values(
            ($doc//tei:text//tei:name[@type=$register]/@ref,
            $doc//tei:text//tei:rs[@type=$register]/@ref,
            if ($register = "person") then
                $doc//node()/@resp
            else (),
            if ($register = "art") then
                $doc//tei:graphic/@corresp
            else
                ()))
    let $entities :=
        for $entity in $all-entities return
            tokenize($entity, " ")
    let $register-doc := doc($config:register || $register || ".xml")
    let $register-entities := $register-doc//index:entry[not(index:error)]/@index:ref
    return
        let $resolved-entities :=
            for $entity in $entities return
                if (matches($entity, '^#')) then
                    $doc//.[@xml:id=replace($entity, '#', '')]/@ref
                else
                    $entity
        let $new-entities := functx:value-except($resolved-entities, $register-entities)
        let $new-entries := kmg-register-transformation:get-new-entities($new-entities, $register, $uri)
        return
            if (count($new-entries) > 0) then
                (kmg-register-transformation:remove-resolved-error($register-doc),
                update insert $new-entries into $register-doc//index:body,
                util:log-system-out(util:system-time() || ": added any new entities to " || $register || "."))
            else
                util:log-system-out(util:system-time() || ": no new entities found for " || $register || ".")
};
